# --8<--8<--8<--8<--
#
# Copyright (C) 2006 Smithsonian Astrophysical Observatory
#
# This file is part of mst_envs
#
# mst_envs is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# mst_envs is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#       Free Software Foundation, Inc.
#       51 Franklin Street, Fifth Floor
#       Boston, MA  02110-1301, USA
#
# -->8-->8-->8-->8--

use strict;
use warnings;

use File::Spec::Functions;

use lib '/proj/axaf/simul/lib/perl5';

use CXC::SysArch;

our ( $OS, $OSrev, $machine, $buildtarget, $specifictarget );

sub compact_paths
{
  my $ sep = shift;

  my @paths = map { chomp( my $x = ref $_ ? catdir( @$_ ) : $_); split( /:/, $x ) } @_;

  my %in;
  join( $sep, grep { $_ && ! $in{$_}++ } @paths );
}

# takes a list of paths. if a path is an arrayref, calls catdir() on
# its contents
sub resolve_paths
{
  map { resolve_path( ref $_ ? catdir(@$_) : $_ )  }  @_;
}

###############################################
#
# Follow links:         %l|%L
#   An attempts to resolve links is made.
# Do not follow links:  %p|%P
#   No attempt to resolve links is made.
# Unique:               %L|%P
#   Only first matching directory is returned.
# Not unique:           %l|%p
#   All matching directories are returned.


sub resolve_path {

  my ( $path ) = @_;

  my ( $pattern ) = $path =~ /(%l|%L|%p|%P)/;

  return ( $path ) unless $pattern;

  my $unique = $pattern eq "\U$pattern";  # unique if any uppercase patterns.
  my $follow_link = "\L$pattern" eq '%l'; # follow links if either %l or %L.
  my $pos = index( $path, $pattern );     # find start of pattern in path.

  # change pattern to %s for ease of use(think sprintf).
  $path =~ s/$pattern/%s/;

  # figure out where to check.
  my @root = split( /:/, $CXC::SysArch::Vars{platforms} );

  my $dir = substr( $path, 0, $pos + 2 );
  my $rest = substr( $path, $pos + 2 );

  my @paths;
  my $real_path;
  foreach ( @root )
  {
    $real_path = sprintf( $dir, $_ );

    if ( -l $real_path )
    {
      next unless $follow_link;

      my $loop;
      for ( $loop = 0 ; $loop < 10 && -l $real_path ; $loop++ )
      {
	my $new_path = readlink( $real_path );
	if ( $new_path =~ m:^/: )
	{
	  $real_path = $new_path;
	}
	else
	{
	  $real_path =~ s:/[^/]*$:/$new_path:;
	}
      }
      do { print ( STDERR "$real_path: too many loops\n" ); next }
       if $loop == 10;
    }

    next unless -d $real_path;

    my $resolved_path = $real_path . $rest;
    next unless -d $resolved_path;

    push @paths, $resolved_path;
    last if $unique;
  }
  return wantarray ? @paths : $paths[0];
}

1;
